
--
-- 2010/11/13   
--

with Ada.Text_IO; 	use Ada.Text_IO;
with Fir_Pak;
with Gnat.Random_Numbers;
with Interfaces;	use Interfaces;
with Png_Pak;
with Direct_IO;

procedure Modelka is

	KRAT      : constant := 4;
	POLYNOM   : constant := 10;
	PSPLEN    : constant := 2 ** POLYNOM;
	AMPLITUDA : constant := 4096.0;
	NOISE     : constant := 2048.0;
	VIBORKA   : constant := 16;

	package Png is new Png_Pak (16#240#);

	type Zapis_Type is array (0 .. PSPLEN - 1) of Float;
	package Zapis_IO is new Direct_IO (Integer_16);

	Z0      : Zapis_Type;
  	Z0_File : Zapis_IO.File_Type;
  	Z1_File : Zapis_IO.File_Type;

	----------------------------------------------------------------------

	procedure Gener is
		G1 : Gnat.Random_Numbers.Generator;

		type FIR_IDX is new Integer range 1 .. 33;
		type FIR_ARR is array (FIR_IDX) of Float;
		Fir_Coef : constant FIR_ARR :=
		(	 1.221825038913e-018,  0.001421994464405 , -1.168883823677e-018, -0.0009410664166939,
			 4.324216582989e-034, -0.001639437043496 ,  2.563371878421e-018,  0.007889680954298 ,
			-6.398863810409e-018, -0.02013351181093  ,  1.0931900964e-017  ,  0.04305487389537  ,
			-1.52501378298e-017 , -0.0926113811686   ,  1.835875537721e-017,  0.3136110402062   ,
			 0.5                ,  0.3136110402062   ,  1.835875537721e-017, -0.0926113811686   ,
			-1.52501378298e-017 ,  0.04305487389537  ,  1.0931900964e-017  , -0.02013351181093  ,
			-6.398863810409e-018,  0.007889680954298 ,  2.563371878421e-018, -0.001639437043496 ,
			-4.324216582989e-034, -0.0009410664166939, -1.168883823677e-018,  0.001421994464405 , 1.221825038913e-018);
		package Fir0 is new FIR_PAK (INDEX => FIR_IDX, FIR_TYP => FIR_ARR, COEF => FIR_COEF, ITEM_TYP => Float, CLEAR => 0.0);
		package Fir1 is new FIR_PAK (INDEX => FIR_IDX, FIR_TYP => FIR_ARR, COEF => FIR_COEF, ITEM_TYP => Float, CLEAR => 0.0);

		procedure Mak0 (B : Boolean) is
			Mul  : Float;
			procedure Mak1 (E : Float) is
				procedure Mak2 (E : Float) is
					W16  : Integer_16;
				begin
					W16 := Integer_16 (E);
					Zapis_IO.Write (Z0_File, Integer_16 (E + Gnat.Random_Numbers.Random_Gaussian (G1) * NOISE));
				end Mak2;
			begin
				Mak2 (Fir1.Go (E));
				Mak2 (Fir1.Go (0.0));
			end Mak1;
		begin
			if B then Mul := AMPLITUDA; else Mul := -AMPLITUDA; end if;
			for I in Zapis_Type'Range loop
				Mak1 (Fir0.Go (Z0 (I) * Mul));
				Mak1 (Fir0.Go (0.0));
			end loop;
		end Mak0;
	begin
		Png.Reset;
		Fir0.Reset;
		Fir1.Reset;
		Gnat.Random_Numbers.Reset (G1);
		Zapis_IO.Create (Z0_File, Zapis_IO.Out_File, "a.pcm");
		for I in 1 .. VIBORKA / 2 * PSPLEN loop
			for J in 1 .. KRAT loop
				Zapis_IO.Write (Z0_File, Integer_16 (Gnat.Random_Numbers.Random_Gaussian (G1) * Float (NOISE)));
			end loop;
		end loop;
		for I in 1 .. VIBORKA loop
			Mak0 (Png.Go);
		end loop;
		Zapis_IO.Close (Z0_File);
	end Gener;

	-----------------------------------------------------------------------

	procedure Corelator is
		type Buf_Type is array (0 .. PSPLEN * KRAT - 1) of Float;
		Buf : Buf_Type := (others => 0.0);
		W16 : Integer_16;
		Sum : Float;
	begin

		Zapis_IO.Open (Z0_File, Zapis_IO.In_File, "a.pcm");
		Zapis_IO.Create (Z1_File, Zapis_IO.Out_File, "b.pcm");
		declare
			use Zapis_IO;
			Len  : Zapis_IO.Count := Size (Z0_File);
			Step : Zapis_IO.Count := Len / 100;
			Cnt  : Zapis_IO.Count := 0;
		begin
			if Step = 0 then Step := 1; end if;
			while not Zapis_IO.End_Of_File (Z0_File) loop
				Zapis_IO.Read (Z0_File, W16);
				Cnt := Cnt + 1;
				if Cnt mod Step = 0 then
					Put (ASCII.CR & Zapis_IO.Count'Image (100 * Cnt / Len));
				end if;
				Buf := Buf (Buf'First + 1 .. Buf'Last) & Float (W16);
				Sum := 0.0;
				for I in Zapis_Type'Range loop
					Sum := Sum + Buf (I * KRAT) * Z0 (I);
				end loop;
				Zapis_IO.Write (Z1_File, Integer_16 (Sum / Float (PSPLEN) * 10.0));
			end loop;
		end;
		Zapis_IO.Close (Z1_File);
		Zapis_IO.Close (Z0_File);
	end Corelator;

	-----------------------------------------------------------------------

begin -- Modelka --

	Put_Line ("Start");

	--  
	Png.Reset;
	for I in Zapis_Type'Range loop
		if Png.Go then Z0 (I) := +1.0; else Z0 (I) := -1.0; end if;
	end loop;

	-- 
	Gener;

	-- 
	Corelator;

	Put_Line ("Ok!");

end Modelka;
